#![warn(clippy::all, clippy::pedantic)]
#![doc = include_str!("../README.md")]

mod decode;
mod encode;
mod error;

pub use {decode::Decoder, encode::Encoder, error::Error as B64Error};

#[derive(Clone, Copy)]
/// Defines the character set used for encoding and decoding
pub struct B64Alphabet {
    items: [char; 64],
    pad: char,
}

/// The most common Base64 alphabet as defined in [RFC4648](https://www.rfc-editor.org/rfc/rfc4648)
pub static B64_RFC4648_ALPHABET: B64Alphabet = B64Alphabet {
    items: [
        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R',
        'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
        'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1',
        '2', '3', '4', '5', '6', '7', '8', '9', '+', '/',
    ],
    pad: '=',
};

impl Default for B64Alphabet {
    fn default() -> Self {
        B64_RFC4648_ALPHABET
    }
}

impl B64Alphabet {
    pub(crate) fn get(&self, c: char) -> Option<usize> {
        for (idx, x) in self.items.iter().enumerate() {
            if *x == c {
                return Some(idx);
            }
        }
        None
    }

    pub(crate) fn pad(&self) -> char {
        self.pad
    }
}
